{-------------------------------------------------------------------------------

The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

     The Original Code is FileManagers.pas released at May 26th, 2007.
     The Initial Developer of the Original Code is Priyatna.
     (Website: http://www.priyatna.org Email: me@priyatna.org)
     All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

-------------------------------------------------------------------------------}

unit FileManagers;

interface

{ $DEFINE INDONESIAN}

uses
  Windows, Classes, Controls, Forms, SysUtils, Dialogs, Menus, IniFiles;

const
  MRUSection = 'MostRecentFiles';

type
  TOnNewEvent = function (Sender: TObject): Boolean of object;
  TOnOpenSaveEvent = function (Sender: TObject; FileName: TFileName): Boolean of object;

  TFileManager = class(TComponent)
  private
    MRUList: TStringList;
    FFileName: TFileName;
    FOnChangeFile: TNotifyEvent;
    FOpenDialog: TOpenDialog;
    FSaveDialog: TSaveDialog;
    FOnOpen: TOnOpenSaveEvent;
    FOnNew: TOnNewEvent;
    FOnSave: TOnOpenSaveEvent;
    FOnModified: TNotifyEvent;
    FModified: Boolean;
    FMaxMRUCount: Integer;
    procedure SetModified(const Value: Boolean);
    function InternalSave: Boolean;
    procedure SendStatus;
    function CheckModified: Boolean;
    procedure AddToMRU(AFileName: string);
    procedure MRUHandler(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure New;
    procedure Open;
    procedure Save;
    procedure SaveAs;
    function Close: TCloseAction;
    procedure LoadMRUFromIni(Ini: TIniFile; Section: string = MRUSection);
    procedure SaveMRUToIni(Ini: TIniFile; Section: string = MRUSection);
    procedure LoadMRUFromFile(FileName: TFileName);
    procedure SaveMRUToFile(FileName: TFileName);
    procedure ShowMRUList(Menu: TMenuItem);
    procedure ManualOpen(AFileName: TFileName);
    procedure ManualSave(AFileName: TFileName);
    property FileName: TFileName read FFileName;
    property Modified: Boolean read FModified write SetModified;
  published
    property MaxMRUCount: Integer read FMaxMRUCount write FMaxMRUCount;
    property OpenDialog: TOpenDialog read FOpenDialog write FOpenDialog;
    property SaveDialog: TSaveDialog read FSaveDialog write FSaveDialog;
    property OnChangeFile: TNotifyEvent read FOnChangeFile write FOnChangeFile;
    property OnModified: TNotifyEvent read FOnModified write FOnModified;
    property OnNew: TOnNewEvent read FOnNew write FOnNew;
    property OnOpen: TOnOpenSaveEvent read FOnOpen write FOnOpen;
    property OnSave: TOnOpenSaveEvent read FOnSave write FOnSave;
  end;

  procedure Register;

implementation


const
  MRUTagId = $19071977;

  {$IFDEF INDONESIAN}
  AskToSaveStr = 'File ini telah berubah. Anda akan menyimpannya terlebih dahulu?';
  FileNotFoundStr = 'File ini tidak dapat ditemukan. Mungkin telah dihapus atau dipindahkan.';
  {$ELSE}
  AskToSaveStr = 'File was modified. Do you want to save it first?';
  FileNotFoundStr = 'Unable to locate the file. It might be deleted or moved.';
  {$ENDIF}


procedure Register;
begin
  RegisterComponents('priyatna.org', [TFileManager]);
end;


{ TFileManager }

constructor TFileManager.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  MRUList := TStringList.Create;
  FModified := False;
  FFileName := '';
  FMaxMRUCount := 10;
end;

destructor TFileManager.Destroy;
begin
  MRUList.Free;
  inherited Destroy;
end;

procedure TFileManager.New;
begin
  if not CheckModified
    then Exit;
  if Assigned(FOnNew)
    then FModified := FOnNew(Self)
    else FModified := False;
  FFileName := '';
  SendStatus;
end;

procedure TFileManager.Open;
begin
  if not CheckModified
    then Exit;
  FOpenDialog.InitialDir := ExtractFileDir(FOpenDialog.FileName);
  FOpenDialog.FileName := '';
  if FOpenDialog.Execute
    then ManualOpen(FOpenDialog.FileName);
end;

procedure TFileManager.ManualOpen(AFileName: TFileName);
begin
  if not FileExists(AFileName) then
  begin
    MessageDlg(FileNotFoundStr + #13#10 + AFileName, mtError, [mbOk], 0);
    Exit;
  end;
  if not Assigned(FOnOpen)
    then Exit;
  if not FOnOpen(Self, AFileName)
    then Exit;
  FFileName := AFileName;
  FModified := False;
  AddToMRU(FFileName);
  SendStatus;
end;

procedure TFileManager.ManualSave(AFileName: TFileName);
begin
  if not Assigned(FOnSave)
    then Exit;
  if not FOnSave(Self, AFileName)
    then Exit;
  FFileName := AFileName;
  FModified := False;
  AddToMRU(FFileName);
  SendStatus;
end;

procedure TFileManager.Save;
begin
  InternalSave;
end;

procedure TFileManager.SaveAs;
begin
  FSaveDialog.InitialDir := ExtractFileDir(FSaveDialog.FileName);
  FSaveDialog.FileName := '';
  if not FSaveDialog.Execute
    then Exit;
  if not Assigned(FOnSave)
    then Exit;  
  if not FOnSave(Self, FSaveDialog.FileName)
    then Exit;
  FFileName := FSaveDialog.FileName;
  FModified := False;
  AddToMRU(FFileName);
  SendStatus;
end;

function TFileManager.Close: TCloseAction;
begin
  if CheckModified
    then Result := caHide
    else Result := caNone;
end;

procedure TFileManager.SetModified(const Value: Boolean);
begin
  if (FModified <> Value) then
  begin
    FModified := Value;
    if Assigned(FOnModified)
      then FOnModified(Self);
  end;  
end;

function TFileManager.InternalSave: Boolean;
begin
  Result := False;
  if (FFileName = '') then
  begin
    FSaveDialog.InitialDir := ExtractFileDir(FSaveDialog.FileName);
    FSaveDialog.FileName := '';
    if not FSaveDialog.Execute
      then Exit;
    FFileName := FSaveDialog.FileName;
  end;
  if not Assigned(FOnSave)
    then Exit;
  if not FOnSave(Self, FFileName)
    then Exit;
  FModified := False;
  AddToMRU(FFileName);
  SendStatus;
  Result := True;
end;

procedure TFileManager.SendStatus;
begin
  if Assigned(FOnModified)
    then FOnModified(Self);
  if Assigned(FOnChangeFile)
    then FOnChangeFile(Self);
end;

function TFileManager.CheckModified: Boolean;
begin
  Result := True;
  if FModified then
  begin
    case MessageDlg(AskToSaveStr, mtWarning, mbYesNoCancel, 0) of
      mrYes: Result := InternalSave;
      mrNo: ;
      mrCancel: Result := False;
    end;
  end;
end;

procedure TFileManager.SaveMRUToIni(Ini: TIniFile; Section: string = MRUSection);
var
  x: Integer;
begin
  if Ini.SectionExists(Section)
    then Ini.EraseSection(Section);
  Ini.WriteInteger(Section, 'Count', MRUList.Count);
  for x := 0 to MRUList.Count-1
    do Ini.WriteString(Section, 'Item' + IntToStr(x), MRUList[x]);
end;

procedure TFileManager.LoadMRUFromIni(Ini: TIniFile; Section: string = MRUSection);
var
  x, n: Integer;
  Str: string;
begin
  MRUList.Clear;
  n := Ini.ReadInteger(Section, 'Count', 0);
  for x := 0 to n-1 do
  begin
    Str := Ini.ReadString(Section, 'Item' + IntToStr(x), '');
    if (Str <> '')
      then MRUList.Add(Str);
  end;
end;

procedure TFileManager.SaveMRUToFile(FileName: TFileName);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(FileName);
  try
    SaveMRUToIni(Ini, MRUSection);
  finally
    Ini.Free;
  end;
end;

procedure TFileManager.LoadMRUFromFile(FileName: TFileName);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(FileName);
  try
    LoadMRUFromIni(Ini, MRUSection);
  finally
    Ini.Free;
  end;
end;

procedure TFileManager.AddToMRU(AFileName: string);
var
  x: Integer;
begin
  for x := MRUList.Count-1 downto 0 do
  begin
    if SameText(MRUList[x], AFileName)
      then MRUList.Delete(x);
  end;
  if (FMaxMRUCount > 0) then
  begin
    while (MRUList.Count > FMaxMRUCount)
      do MRUList.Delete(MRUList.Count-1);
  end;
  MRUList.Insert(0, AFileName);
end;

procedure TFileManager.MRUHandler(Sender: TObject);
var
  Idx: Integer;
  Str: string;
begin
  if not CheckModified
    then Exit;
  Idx := (Sender as TComponent).Tag;
  Str := MRUList[Idx];
  if not FileExists(Str) then
  begin
    MessageDlg(FileNotFoundStr + #13#10 + Str, mtError, [mbOk], 0);
    MRUList.Delete(Idx);
    Exit;
  end;
  if not Assigned(FOnOpen)
    then Exit;
  if not FOnOpen(Self, Str)
    then Exit;
  FFileName := Str;
  FModified := False;
  AddToMRU(FFileName);
  SendStatus;
end;

procedure TFileManager.ShowMRUList(Menu: TMenuItem);
var
  x: Integer;
  m: TMenuItem;
begin
  for x := Menu.Count-1 downto 0 do
  begin
    if (Menu.Items[x].HelpContext = MRUTagId)
      then Menu.Items[x].Free;
  end;
  for x := 0 to MRUList.Count-1 do
  begin
    m := TMenuItem.Create(nil);
    m.Caption := IntToHex(x + 1, 1) + ' - ' + StringReplace(MRUList[x], '&', '&&', [rfReplaceAll]);
    m.HelpContext := MRUTagId;
    m.Tag := x;
    m.OnClick := MRUHandler;
    Menu.Add(m);
  end;
end;

end.
